package org.jcomet.dynamic;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;

import org.jcomet.core.FileChangeListener;
import org.jcomet.core.FileMonitor;

/**
 * This object can monitor its source configuration file and reload when even it is modified. It is also provides the ability to register listeners that are notified when properties change.
 * 
 * @author Amir Shevat
 *
 */
public class DynamicProperties extends Properties implements Reloadable, FileChangeListener {
	

	private static final long serialVersionUID = 8303688016890267194L;
	/**
	 * Maps the files that are the source of these properties to their monitors
	 */
	private HashMap filesToFileMonitor = new HashMap();
	/**
	 * The files that are the source of these properties
	 */
	private ArrayList files= new ArrayList();
	
	private PropertyChangedListener propertyChangedListener;
	
	/**
	 * Creates a DynamicProperties that does not monitor its source file. User can still reload when ever there is a need.
	 */
	public void load(File propFile) throws IOException{
		load(propFile,  -1);
	}
	
	/**
	 * Once loaded this object will monitor and reload every change to its source file.
	 * @param monitorInterval -1 for "do not monitor", greater then 0 for "monitor every monitorInterval" (milli)
	 */
	public void load(File propFile, long monitorInterval) throws IOException{
		super.load(new FileInputStream(propFile));
		if(filesToFileMonitor.containsKey(propFile)){
			FileMonitor fm =(FileMonitor) filesToFileMonitor.remove(propFile);
			fm.stop();
			
		}
		if(monitorInterval>0){
			FileMonitor fm = new FileMonitor(propFile, this,monitorInterval );
			filesToFileMonitor.put(propFile, fm);
		}
		
		files.add(propFile);
	}
	
	/**
	 * Forces this Object to reload all its source files and notify the listener of any change in them
	 */
	public synchronized void reload() {
		Iterator filesIter = files.iterator();
		while (filesIter.hasNext()) {
			File file = (File) filesIter.next();
			try {
				load(new FileInputStream(file));
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}//reload()
	
	/**
	 * same as super but with a notification addon
	 */
	public synchronized Object put(Object key, Object value) {
		Object oldVal = get(key);
		if(containsKey(key) && !oldVal.equals(value) ){
			// this is a changed var
			if(propertyChangedListener!=null){
				propertyChangedListener.propertyChanged((String)key,(String)oldVal, (String)value );
			}
		}
		return super.put(key, value);
	}

	/**
	 * Internal use - do not invoke
	 */
	public void fileChanged(File file) throws  IOException {
		load(new FileInputStream(file));
		
	}
	
	/**
	 * Internal use - do not invoke
	 */
	protected void finalize() throws Throwable {
		Iterator filesMonIter = filesToFileMonitor.values().iterator();
		while (filesMonIter.hasNext()) {
			FileMonitor fileMon = (FileMonitor) filesMonIter.next();
			fileMon.stop();
		}
		super.finalize();
	}
	
	/**
	 * Returns the PropertyChangedListener of this object
	 * @return the PropertyChangedListener of this object
	 */
	public PropertyChangedListener getPropertyChangedListener() {
		return propertyChangedListener;
	}
	
	/**
	 * Sets the PropertyChangedListener of this object, application should use this method to register themselves as listeners 
	 * @param propertyChangedListener PropertyChangedListener of this object
	 */
	public void setPropertyChangedListener(
			PropertyChangedListener propertyChangedListener) {
		this.propertyChangedListener = propertyChangedListener;
	}

}
